package com.loafle.overflow.module.member.service;

import com.loafle.overflow.module.email.service.EmailAuthService;
import com.loafle.overflow.module.member.dao.MemberDAO;
import com.loafle.overflow.module.member.exception.*;
import com.loafle.overflow.module.member.model.Member;
import com.loafle.overflow.module.meta.model.MetaMemberStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;

/**
 * Created by geek on 17. 6. 28.
 */
@Service("MemberService")
public class MemberService {

    @Autowired
    private MemberDAO memberDAO;

    @Autowired
    private EmailAuthService emailAuthService;

    private BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();

    public Member signin(String signinId, String signinPw) throws SignInIdNotExistException, EmailNotConfirmedException, SignInPwNotMatchException {
        Member m = this.memberDAO.findByEmail(signinId);

        if ( null == m ) {
            throw new SignInIdNotExistException();
        }

        if ( m.getStatus().getId() == 1 ) {
            throw new EmailNotConfirmedException();
        }

        Boolean match = passwordEncoder.matches(signinPw, m.getPw());
        if(!match) {
            throw new SignInPwNotMatchException();
        }

        return m;
    }

    public Member signup(Member member, String pw) {

        Member isMember = this.memberDAO.findByEmail(member.getEmail());

        if (null != isMember && isMember.getId() > 0) {
            throw new JoinedEmailException();
        }

        member.setPw(passwordEncoder.encode(pw));

        if (member.getStatus() == null) {
            member.setStatus(new MetaMemberStatus((short) 1));
        }

        Member resMember = this.memberDAO.save(member);

        this.emailAuthService.sendEmailByMember(member.getId(), member.getEmail());

        return resMember;
    }

    public void signout(Member member) {
        // Todo websocket session remove
    }

    public Member modify(Member member, String pw) {

        Member preMember = this.memberDAO.findByEmail(member.getEmail());

        if (null != pw && !pw.equals("")) {
            Boolean match = passwordEncoder.matches(member.getPw(), preMember.getPw());
            if(!match) {
                member.setPw(passwordEncoder.encode(pw));
            }
        } else {
            member.setPw(preMember.getPw());
        }

        if (member.getStatus() == null || member.getStatus().getId() < 0) {
            member.getStatus().setId(preMember.getStatus().getId());
        }
        return this.memberDAO.save(member);
    }

    public Member confirmPw(String signinId, String signinPw) {

        Member preMember = this.memberDAO.findByEmail(signinId);

        String encodePw = passwordEncoder.encode(signinPw);
        Boolean match = passwordEncoder.matches(encodePw, preMember.getPw());

        if (!match) {
            throw new SignInPwNotMatchException();
        }

        return preMember;
    }

    public Member forgotPassword(String signinId, String newPw) {
        Member preMember = this.memberDAO.findByEmail(signinId);

        if (null == preMember) {
            throw new SignInIdNotExistException();
        }

        preMember.setPw(this.passwordEncoder.encode(newPw));
        this.memberDAO.save(preMember);

        return preMember;
    }

    public Member read(long memberId) {
        if (memberId <= 0) {
            // Todo MemberId null Exception
        }

        Member resMember = this.memberDAO.findOne(memberId);


        return resMember;
    }

    public void withdrawal(Member member) {
        // Todo DB delete?
    }

    public void emailConfirm(Member member) {

    }
}
