package com.loafle.overflow.module.sensor.service;

import com.loafle.overflow.commons.model.PageParams;
import com.loafle.overflow.commons.utils.PageUtil;
import com.loafle.overflow.module.domain.model.Domain;
import com.loafle.overflow.module.infra.model.Infra;
import com.loafle.overflow.module.infra.service.InfraService;
import com.loafle.overflow.module.meta.model.MetaSensorStatus;
import com.loafle.overflow.module.probe.model.Probe;
import com.loafle.overflow.module.probe.service.ProbeService;
import com.loafle.overflow.module.sensor.dao.SensorDAO;
import com.loafle.overflow.module.sensor.model.Sensor;
import com.loafle.overflow.module.sensor.model.SensorItem;
import com.loafle.overflow.module.target.model.Target;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.List;

/**
 * Created by insanity on 17. 6. 28.
 */
@Service("SensorService")
public class SensorService  {

    @Autowired
    SensorDAO sensorDAO;

    @Autowired
    private ProbeService probeService;

    @Autowired
    private InfraService infraService;

    @Autowired
    private SensorItemService sensorItemService;

    public Sensor regist(Sensor sensor) {
        return this.sensorDAO.save(sensor);
    }

//    public List<Sensor> readAllByTarget(Target target, PageParams pageParams) {
//        return this.sensorDAO.findAllByTarget(target, PageUtil.getPageRequest(pageParams));
//    }

    public Page<Sensor> readAllByDomain(Domain domain, PageParams pageParams) {

        List<Probe> probeList = this.probeService.readAllByDomain(domain);

        List<Target> targetList = this.infraService.readAllTargetByProbeList(probeList);

        return this.sensorDAO.findAllByTargetList(targetList, PageUtil.getPageRequest(pageParams));
    }

    public Page<Sensor> readAllByInfra(long infraId, PageParams pageParams) {
        Infra dbInfra = this.infraService.read(infraId);

        if(dbInfra == null || dbInfra.getTarget() == null) return null;

        return this.sensorDAO.findAllByTarget(dbInfra.getTarget(), PageUtil.getPageRequest(pageParams));

    }

    public Sensor read(String id) {
        return this.sensorDAO.findOne(Long.valueOf(id));
    }

    public void remove(Sensor sensor) {
        this.sensorDAO.delete(sensor);
    }

    public Sensor start(Sensor sensor) {
        MetaSensorStatus status = new MetaSensorStatus((short)1);
        sensor.setStatus(status);
        return this.sensorDAO.save(sensor);
    }

    public Sensor stop(Sensor sensor) {
        MetaSensorStatus status = new MetaSensorStatus((short)2);
        sensor.setStatus(status);
        return this.sensorDAO.save(sensor);
    }

    @Transactional
    public Sensor registSensorConfig(Sensor sensor, List<SensorItem> sensorItemList) {

        this.sensorDAO.save(sensor);

        for(SensorItem sensorItem : sensorItemList) {
            sensorItem.setSensor(sensor);
        }

        this.sensorItemService.registAll(sensorItemList);

        return sensor;
    }
}
